/**
 * Functions for all the basic components of the Zeedhi-Framework
 * @param {ProtractorBrowser} browser
 * @param {Browser} protractor
 * @constructor
 */
function FrameworkFunctionalTestApi() {
    var ec = protractor.ExpectedConditions;
    var self = this;

    this.KEY = protractor.Key;

    this.ORDER_NONE = 0;
    this.ORDER_ASC = 1;
    this.ORDER_DESC = 2;
    var ORDER_NONE_STR = '';
    var ORDER_SYMBOLS = ['', '▼', '▲'];

    /**
     * Adds the locator 'by.cssWithExactText'. This locator find elements by CSS that has the exact text.
     * @param {String} elementSelector - The selector of the element.
     * @param {String} searchString - Text to be search.
     * @return {Locator} The element with especified {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     */
    by.addLocator('cssWithExactText', function(elementSelector, searchString) {
        var nodelist = document.querySelectorAll(elementSelector);
        return Array.prototype.filter.call(nodelist, function(node) {
            return node.outerText === searchString.toString();
        });
    });

    /**
     * Press a key from the keyboard.
     * @param {String} key - The key to be sent.
     */
    this.pressKey = function(key) {
        browser.actions().sendKeys(key).perform();
    };

    /**
     * Validate the optional parameter, giving then the defaltValue only if it's undefined.
     * @param parameter - The parameter to be tested.
     * @param defaultValue - The parameter's default value.
     * @returns The parameter validated.
     */
    this.validateOptionalParameter = function(parameter, defaultValue) {
        return parameter === undefined ? defaultValue : parameter;
    };

    /**
     * Config the given locator with the popup prefix.
     * @param {String} locator - The locator to be configured.
     * @param {Boolean} isInsidePopup - Boolean indicating if the locator refers to an element inside a popup.
     * @returns {String} The string for the locator with the especified configuration.
     */
    this.setPopupPrefix = function(locator, isInsidePopup) {
        return isInsidePopup ? '.popup-container ' + locator : locator;
    };

    /**
     * Get the NotificationMessage text by it's index.
     * @param {Number} key - The index of the notification starting at 0 (top-down order).
     * @return {Promise.<String>} A promise that will be resolved with the notification text.
     */
    this.getNotificationTextByIndex = function(index) {
        var notificationElements = element.all(by.css('.zh-notification-messages span.notification-message-text'));
        return notificationElements.get(index).getText();
    };

    /**
     * Get the text of the bottom-most NotificationMessage.
     * @return {Promise.<String>} A promise that will be resolved with the notification text.
     */
    this.getLastNotificationText = function() {
        var notificationElements = element.all(by.css('.zh-notification-messages span.notification-message-text'));
        return notificationElements.last().getText();
    };

    /**
     * Fill the given field with the especified value.
     * @param {String} fieldName - The field name.
     * @param {String | Number} value - The value to be written on the field.
     * @param {Boolean} [clearPreviousValue=true] Option to clear the previous value on the field.
     */
    this.fillField = function(fieldName, value, clearPreviousValue) {
        clearPreviousValue = this.validateOptionalParameter(clearPreviousValue, true);
        var field = element(by.id(fieldName));
        browser.wait(ec.presenceOf(field));
        if (clearPreviousValue) {
            field.clear();
        }
        field.sendKeys(value);
    };

    /**
     * Determine whether the splash is visible on the page.
     * @returns {Promise.<Boolean>} A promise that will be resolved with the result of the splash's visibility.
     */
    this.splashInvisible = function() {
            return ec.invisibilityOf(element(by.id('splash')));
    };

    /**
     * Determine whether the splash is not active and the condition is true.
     * @param {Boolean} condition - Condition to be evaluated with the visibility of the splash.
     * @returns {Promise.<Boolean>} A promise that will be resolved with the logical result of the condition AND the visibility of the splash.
     */
    this.splashNotActiveAnd = function(condition) {
        return ec.and(this.splashInvisible(), condition);
    };

    /**
     * Determine whether the overlay is visible on the page.
     * @returns {Promise.<Boolean>} A promise that will be resolved with the logical result of the overlay's visibility.
     */
    this.overlayInvisible = function() {
        return ec.invisibilityOf(element(by.css('div.zh-background-loading')));
    };

    /**
     * Creates an waits condition to check if the splash is in first plan.
     * @param {Boolean | Function} condition - Condition to 'and' together with the overlay visibility.
     * @returns {Promise.<Boolean>} A promise that will be resolved with the logical result of the condition AND the visibility of the overlay.
     */
    this.overlayNotActiveAnd = function(condition) {
        return ec.and(this.overlayInvisible(), condition);
    };

    /**
     * Waits until any loading screen used by Zeedhi don't be neither active neither visible.
     */
    this.waitZhLoading = function() {
        browser.wait(this.overlayNotActiveAnd(this.splashInvisible()));
    };

    /**
     * Creates an waits condition to check if the element is clickable.
     * @param {Element} element - The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element} to have it's click functionality tested.
     * @returns {Promise.<Boolean>} A promise that will be resolved with a boolean telling if the element is clickable.
     */
    this.elementClickAble = function(element) {
        return this.splashNotActiveAnd(this.overlayNotActiveAnd(ec.elementToBeClickable(element)));
    };

    /**
     * Click on the given element.
     * @param {Element} element - The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element} to be clicked.
     */
    this.clickByElementFinder = function(element) {
        browser.wait(this.elementClickAble(element));
        element.click();
    };

    /**
     * Click on the element with the especified locator.
     * @param {Locator} elementLocator- The element with especified {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     */
    this.clickElement = function(elementLocator) {
        this.clickByElementFinder(element(elementLocator));
    };

    /**
     * Click on the field with the given name.
     * @param {String} fieldName - The field name.
     */
    this.clickField = function(fieldName) {
        this.clickElement(by.css('.zh-field > div > [id="' + fieldName + '"]'));
    };

    /**
     * Creates an waits condition to get the title of the Zeedhi Container.
     * @return {Promise.<String>} A promise that will be resolved with the Container title.
     */
    this.getContainerTitle = function() {
        return element(by.className("zh-header-title")).getText();
    };

    /**
     * Creates an waits condition to get the subtitle of the Zeedhi Container.
     * @return {Promise.<String>} A promise that will be resolved with the Container subtitle.
     */
    this.getContainerSubtitle = function() {
        return element(by.className("zh-header-info")).getText();
    };

    /**
     * Creates an waits condition to get the title of the Zeedhi Page.
     * @return {Promise.<String>} A promise that will be resolved with the container title.
     */
    this.getPageTitle = function() {
        //return browser.get(browser.params.applicationUrl).then(function() {
			//return browser.getTitle();
		//});			
		browser.getTitle().then(function(title) {
			console.log(title);
		});
    };

    /**
     * Creates an waits condition to get the title of the Zeedhi Popup.
     * @return {Promise.<String>} A promise that will be resolved with the Popup title.
     */
    this.getPopupTitle = function() {
        return element(by.className('popup-title')).getText();
    };

    /**
     * Creates an waits condition to get the text of the Zeedhi Alert.
     * @return {Promise.<String>} A promise that will be resolved with the alert text.
     */
    this.getAlertText = function() {
        var alert = element(by.className('zh-text-alert'));
        return alert.getText();
    };

    /**
     * Click on the action with the especified icon and region.
     * @param {String} actionRegionClass - The region of the action.
     * @param {String} actionLabel - The label of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickActionByRegionAndLabel = function(actionRegionClass, actionLabel, isInsidePopup) {
        this.validateOptionalParameter(isInsidePopup, false)
        var actionSelector = this.setPopupPrefix('.' + actionRegionClass + '>ul>li>a>span.zh-footer-title-sprit', isInsidePopup);
        this.clickElement(by.cssWithExactText(actionSelector, actionLabel));
    };

    /**
     * Click on the especified action on the right region.
     * @param {String} actionLabel - The label of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of pop up
     */
    this.clickRightActionByLabel = function(actionLabel, isInsidePopup) {
        this.clickActionByRegionAndLabel('zh-footer-right', actionLabel, isInsidePopup);
    };

    /**
     * Click on the especified action on the center region.
     * @param {String} actionLabel - The label of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of pop up
     */
    this.clickCenterActionByLabel = function(actionLabel, isInsidePopup) {
        this.clickActionByRegionAndLabel('zh-footer-center', actionLabel, isInsidePopup);
    };

    /**
     * Click on the especified action on the left region.
     * @param {String} actionLabel - The label of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of pop up
     */
    this.clickLeftActionByLabel = function(actionLabel, isInsidePopup) {
        this.clickActionByRegionAndLabel('zh-footer-left', actionLabel, isInsidePopup);
    };

    /**
     * Click on the action with the especified label and region.
     * @param {String} actionRegionClass - The region of the action.
     * @param {String} actionIcon - The icon of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickActionByRegionAndIcon = function(actionRegionClass, actionIcon, isInsidePopup) {
        //@todo review this case.
        this.validateOptionalParameter(isInsidePopup, false);
        var actionSelector = this.setPopupPrefix("." + actionRegionClass + ">ul>li>a>span.action-icon>span.zh-icon-" + actionIcon, isInsidePopup);
        var e = element(by.css(actionSelector)).element(by.xpath('ancestor::a'));
        this.clickByElementFinder(e);
    };

    /**
     * Click on the action with the especified icon on the right region.
     * @param {String} actionIcon - The icon of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickRightActionByIcon = function(actionIcon, isInsidePopup) {
        this.clickActionByRegionAndIcon('zh-footer-right', actionIcon, isInsidePopup);
    };

    /**
     * Click on the action with the especified icon on the center region.
     * @param {String} actionIcon - The icon of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickCenterActionByIcon = function(actionIcon, isInsidePopup) {
        this.clickActionByRegionAndIcon('zh-footer-center', actionIcon, isInsidePopup);
    };

    /**
     * Click on the action with the especified icon on the left region.
     * @param {String} actionIcon - The icon of the action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickLeftActionByIcon = function(actionIcon, isInsidePopup) {
        this.clickActionByRegionAndIcon('zh-footer-left', actionIcon, isInsidePopup);
    };

    /**
     * Click on the wizard action with the especified icon and region.
     * @param {String} actionRegionClass - The region of the action.
     * @param {String} actionIcon - The icon of the wizard action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickWizardActionByRegionAndIcon = function(actionRegionClass, actionIcon, isInsidePopup) {
        this.validateOptionalParameter(isInsidePopup, false);
        var e = element(by.css(this.setPopupPrefix("." + actionRegionClass + ">ul>li>a>span.zh-icon-" + actionIcon, isInsidePopup)));
        if (actionIcon == 'plus') {
            e = e.element(by.xpath('ancestor::a'));
        }
        this.clickByElementFinder(e);
    };

    /**
     * Click on the wizard action with the especified icon on the right region.
     * @param {String} actionIcon - The icon of the wizard action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickRightWizardActionByIcon = function(actionIcon, isInsidePopup) {
        this.clickWizardActionByRegionAndIcon('zh-footer-right', actionIcon, isInsidePopup);
    };

    /**
     * Click on the wizard action with the especified icon on the center region.
     * @param {String} actionIcon - The icon of the wizard action.
     * @param {Boolean} [isInsidePopup=false] - Param telling if action is inside of a pop up.
     */
    this.clickCenterWizardActionByIcon = function(actionIcon, isInsidePopup) {
        this.clickWizardActionByRegionAndIcon('zh-footer-center', actionIcon, isInsidePopup);
    };

    /**
     * Click on the striper action with the especified icon and region.
     * @param {String} actionRegionClass - The region of the action.
     * @param {String} actionIcon - The icon of the striper action.
     */
    this.clickStriperActionByRegionAndIcon = function(actionRegionClass, actionIcon) {
        var e = element(by.css(".zh-stripe>section." + actionRegionClass + ">a>span>span.zh-icon-" + actionIcon));
        this.clickByElementFinder(e);
    };

    /**
     * Click on the especified striper action on the right region.
     * @param {String} actionIcon - The icon of the wizard action.
     */
    this.clickRightStriperActionByIcon = function(actionIcon) {
        this.clickStriperActionByRegionAndIcon('right-side', actionIcon);
    };

    /**
     * Determine whether the element is present on the page and wait for it appears.
     * @param {Locator} elementLocator - The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     * @return {Promise.<Boolean>} A Promise that will be resolved with a boolean telling if the element is present or not.
     */
    this.isElementPresent = function(elementLocator) {
        var e = element(elementLocator);
        browser.wait(ec.presenceOf(e));
        return e.isDisplayed();
    };

    /**
     * Determine if the element is present on the page, without wait to it apears.
     * @param {Locator} elementLocator - The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     * @return {Promise.<Boolean>} A Promise that will be resolved with a boolean telling if the element is present or not.
     */
    this.elementExists = function(selector) {
        var e = element(selector);
        return e.isPresent();
    };

    /**
     * Determine if the field is present on the page, without wait to it apears.
     * @param {String} fieldName - The name of the field to check existence.
     * @return {Promise.<Boolean>} A Promise that will be resolved with a boolean telling if the element is present or not.
     */
    this.fieldExists = function(fieldName) {
        return this.elementExists(by.id(fieldName));
    };


    /**
     * Determine whether the field is present and visible on the page and wait for it appears.
     * @param {String} fieldName - The field name.
     * @return {Promise.<Boolean>} A promise that will be resolved with the presency and visibility of the field.
     */
    this.isFieldPresentAndVisible = function(fieldName) {
        return this.isElementPresent(by.id(fieldName));
    };

    /**
     * Open the Zeedhi Menu.
     */
    this.openMenu = function() {
        this.clickElement(by.css('.menu-items>a'));
    };

    /**
     * Returns the locator of the menu with the especified label
     * @param {String} menuLabel - The label of the menu.
     * @return {Locator} The menu {@link https://github.com/angular/protractor/blob/master/docs/locators.md locator}.
     */
    this.getMenuLocatorByLabel = function(menuLabel) {
        return by.cssWithExactText('.menu-title', menuLabel);
    };

    /**
     * Click on the menu with the especified label.
     * @param {String} manuLabel - The menu label.
     */
    this.clickMenuByLabel = function(menuLabel) {
        this.clickElement(this.getMenuLocatorByLabel(menuLabel));
    };

    /**
     * Open the application in the browser.
     */
    this.openApplication = function() {
        browser.get(browser.params.applicationUrl);
    };

    /**
     * Get the value inside the given field on a form view.
     * @param {String} fieldName - The field name.
     * @return {Promise.<String>} A promise that will be resolved with the field's text.
     */
    this.getFieldValueOnFormView = function(fieldName) {
        return element(by.css("#span-field-" + fieldName + ">p")).getText();
    };

    /**
     * Get the value inside the given field on a stripe.
     * @param {String} fieldName - The field name.
     * @return {Promise.<String>} A promise that will be resolved with the field's text.
     */
    this.getFieldValueOnWidgetStripe = function(fieldName) {
        return element(by.css(".stripe-fields>.stripe-row>.stripe-field>.stripe-value-" + fieldName + "")).getText();
    };

    /**
     * Click on the tab of the especified widget.
     * @param {String} widgetLabel - The label of the string.
     */
    this.clickWidgetTab = function(widgetLabel) {
        this.clickElement(by.cssWithExactText('.zh-tab-cell>a', widgetLabel));
    };

    /**
     * Get the number of rows on the especified grid.
     * @param {String} fieldName- The grid's name.
     * @return {Promise.<Integer>} A promise that will be resolved with the number of grid's row.
     */
    this.getNrRowsOnGrid = function(fieldName) {
        return element.all(by.css('.zh-new-grid-' + fieldName + '>.body>div>.tr')).count();
    };

    /**
     * Get the number of rows on the first list
     *
     * @return {Promise.<Integer>} A promise that will be resolved with the number of rows on the list
     */
    this.getNrRowsOnList = function() {
        return element.all(by.css('.zh-template-list > .zh-list-item-label')).count();
    };

    /**
     * Determine whether the menu is present on the page.
     * @param {String} menuLabel - The menu label.
     * @return {Promise.<Boolean>} A promise that will be resolved with the presency of the menu.
     */
    this.isMenuPresent = function(menuLabel) {
        return element(this.getMenuLocatorByLabel(menuLabel)).isPresent();
    };

    /**
     * Select the given date on the calendar.
     * @param {String} dateString- The target data.
     * @param {String} laguage - The language of the page.
     * @example
     * z.selectDateOnCalendar("12/04/1999", "pt_br");
     */
    this.selectDateOnCalendar = function(dateString, language) {
        this.clickDateCalendar(dateString, language);
    };

    /**
     * Click on the especified field before select the given date on the calendar.
     * @param {String} fieldName- The name of the field.
     * @param {String} date - The target data.
     * @param {String} laguage - The language of the page.
     * @example
     * z.selectDateOnCalendar("fieldData", "12/04/1999", "pt_br");
     */
    this.selectDateCalendarField = function(fieldName, date, language) {
        this.clickField(fieldName);
        this.selectDateOnCalendar(date, language);
    };

    /**
     * Click on the especified field before select the given interval date on the calendar.
     * @param {String} fieldName - The name of the field.
     * @param {String} dateBegin - The target begin date.
     * @param {String} dateEnd - The target end date.
     * @param {String} laguage - Language used to especify the month.
     * @example
     * z.selectDateOnCalendar("fieldData", "12/04/1999", "15/06/1999", "pt_br");
     */
    this.selectIntervalDate = function(fieldName, dateBegin, dateEnd, language) {
        this.clickField(fieldName);
        this.selectDateOnCalendar(dateBegin, language);
        this.selectDateOnCalendar(dateEnd, language);
        this.clickRightActionByLabel('OK');
    };

    /**
     * Get the three first letters of each month in the especified language.
     * @param {String} language - The language of the page.
     * @return {Array} The three first letters of each month.
     */
    this.getMonthsByLanguage = function(language) {
        var month = {
            "default": {
                1: 'Jan',
                2: 'Feb',
                3: 'Mar',
                4: 'Apr',
                5: 'May',
                6: 'Jun',
                7: 'Jul',
                8: 'Aug',
                9: 'Sep',
                10: 'Oct',
                11: 'Nov',
                12: 'Dec'
            },
            "pt_br": {
                1: 'Jan',
                2: 'Fev',
                3: 'Mar',
                4: 'Abr',
                5: 'Mai',
                6: 'Jun',
                7: 'Jul',
                8: 'Ago',
                9: 'Set',
                10: 'Out',
                11: 'Nov',
                12: 'Dez'
            }
        };
        return language ? month[language] : month.default;
    };

    /**
     * Click the given date on the calendar.
     * @param {String} dateString - The target data.
     * @param {String} [language] - The language of the page.
     */
    this.clickDateCalendar = function(dateString, language) {
        var dateArray = dateString.split('/');
        var year = dateArray[2];
        var month = parseInt(dateArray[1]);
        var day = parseInt(dateArray[0]);

        this.openCalendarMonth(month, year, language);

        this.clickDayCalendar(parseInt(day));
    };

    /**
     * Goes to the previous decade on the calendar until gets the target.
     * @param {(String|Integer)} year - Target year.
     */
    function goToPreviousDecade(year) {
        element.all(by.binding('year.number')).first().getText().then(function(first) {
            if (year <= first) {
                self.clickElement(by.css('.arrow-left'));
                goToPreviousDecade(year);
            }
        });
    }

    /**
     * Goes to the next decade on the calendar until gets the target.
     * @param {(String|Integer)} year - Target year.
     */
    function goToNextDecade(year) {
        element.all(by.binding('year.number')).last().getText().then(function(last) {
            if (year >= last) {
                self.clickElement(by.css('.arrow-right'));
                goToNextDecade(year);
            }
        });
    }

    /**
     * Click on the especified year on the calendar.
     * @param {(String|Integer)} year - The year to be clicked.
     */
    this.clickYearCalendar = function(year) {
        this.clickElement(by.cssWithExactText('.content', year));
    };

    /**
     * Click on the especified month on the calendar.
     * @param {(String|Integer)} month - The month to be clicked.
     */
    this.clickMonthCalendar = function(month) {
        this.clickElement(by.cssWithExactText('.content', month));
    };

    /**
     * Click on the especified day on the calendar.
     * @param {(String|Integer)} day - The day to be clicked.
     */
    this.clickDayCalendar = function(day) {
        this.clickElement(by.cssWithExactText('div.day:not(.previous-month):not(.next-month):not(.grid)', day));
    };

    /**
     * Click on the spin field number until gets the target number.
     * @param {String} fieldName - The field number name.
     * @param {Integer} targetNumber - The target number.
     */
    this.clickSpinFieldNumber = function(fieldName, targetNumber) {
        var input = element(by.id(fieldName));
        input.getAttribute('value').then(function(currentNumber) {
            if (targetNumber > currentNumber) {
                clickSpinButtonPlusOrMinus('plus', targetNumber, currentNumber, fieldName);
            }
            if (targetNumber < currentNumber) {
                clickSpinButtonPlusOrMinus('minus', currentNumber, targetNumber, fieldName);
            }
        });
    };

    /**
     * Click on the button plus or minus (minuend - subtrahend)  times.
     * @param {String} optionButton- The buton to be clicked. Can be only 'plus' or 'minus'.
     * @param {Integer} minuend - The minuend of subtraction.
     * @param {Integer} subtrahend - The subtrahend of subtraction.
     * @param {String} fieldName - The field number name.
     */
    function clickSpinButtonPlusOrMinus(optionButton, minuend, subtrahend, fieldName) {
        var amountClick = minuend - subtrahend;
        for (var i = 0; i < amountClick; i++) {
            self.clickElement(by.css(".zh-field-" + fieldName + ">.spin-button-" + optionButton));
        }
    }

    /**
     * Determine whether the swipe is present on the page.
     * @return {Promise.<Boolean>} A promise that will be resolved with the presency of the swipe.
     */
    this.isSwipePresent = function() {
        return element(by.css('.zh-swipe-inner')).isPresent();
    };

    /**
     * Get the text inside a column of the row.
     * @param {String} gridId - The grid's id.
     * @param {Integer} row - The row's number. Beggining from 0.
     * @param {Integer} column - The column's number. Beginning from 0.
     * @param {Boolean} [hasCheckbox=false] - Param telling if there are checkboxs on the grid.
     * @returns {Promise.<String>} A promise with the text on grid.
     */
    this.getTextOnGrid = function(gridId, row, column, hasCheckbox) {
        var locator = this.getGridRowColumnLocator(gridId, row, column, hasCheckbox);
        return element(locator).getText();
    };

    /**
     * Get the text inside a property of the row
     *
     * @param {Integer} row        Row number
     * @param {Integer} property   Rroperty number
     *
     * @returns {Promise.<String>} A promise with the text in the list
     */
    this.getTextOnList = function(row, property) {
        var locator = by.css('.zh-template-list > .zh-list-item-label:nth-child(' + (row + 2) + ') > div:nth-child(' + (property + 1) + ') > div > span:nth-child(2)');
        return element(locator).getText();
    };

    /**
     * Click on the especified grid's row's column.
     * @param {String} gridId - The grid's id.
     * @param {Integer} row - The row's number. Beggining from 0.
     * @param {integer} column - The column's number. Beginning from 0.
     * @param {Boolean} [hasCheckbox=false] - Param telling if there are checkboxs on the grid.
     */
    this.clickGridColumn = function(gridId, row, column, hasCheckbox) {
        var locator = this.getGridRowColumnLocator(gridId, row, column, hasCheckbox);
        this.clickElement(locator);
    };

    /**
     * Click on the especified grid's row's column.
     * @param {String} gridId- The grid's id.
     * @param {Integer} row - The row's number. Beggining from 0.
     * @param {Integer} column - The column's number. Beginning from 0.
     * @param {Boolean} [hasCheckbox=false] - Param telling if there are checkboxs on the grid.
     */
    this.getGridRowColumnLocator = function(gridId, row, column, hasCheckbox) {
        hasCheckbox = this.validateOptionalParameter(hasCheckbox, false);
        if (hasCheckbox) {
            return by.css('#grid-' + gridId + '>div.body>.nonbounce>div:nth-child(' + (row + 1) + ')>div:nth-child(' + (column + 2) + ')');
        } else {
            return by.css('#grid-' + gridId + '>div.body>.nonbounce>div:nth-child(' + (row + 1) + ')>div:nth-child(' + (column + 1) + ')');
        }
    };

    /**
     * Check the given grid's row.
     * @param {String} gridId - The grid's Id.
     * @param {Integer} row - The position of the row that will be checked. Beggining from 0.
     */
    this.checkGridRow = function(gridId, row) {
        this.clickGridColumn(gridId, row, -1);
    };

    /**
     * Check the given grid's rows.
     * @param {String} gridId - The grid's Id.
     * @param {Integer[]} rows - The position of the rows that will be checkeds. Beggining from 0.
     */
    this.checkGridRows = function(gridId, rows) {
        for (var row in rows) {
            this.checkGridRow(gridId, rows[row]);
        }
    };

    /**
     * Check all grid's rows.
     * @param {String} [gridId]- The grid's Id.
     */
    this.checkAllGridRows = function(gridId) {
        gridId = this.validateOptionalParameter(gridId, '');
        this.clickElement(by.css('#grid-' + gridId + '>.header>.header-content>.tr>.grid-checkbox-column>div'));
    };

    /**
     * Click on Filter Parameters.
     */
    this.clickFilterParameters = function() {
        this.clickElement(by.cssWithExactText('.zh-label', 'Filter Parameters'));
    };

    /**
     * Click on Clear filters.
     */
    this.clickClearFilters = function() {
        this.clickElement(by.cssContainingText('.zh-popup-title-sprit', 'Clear filters'));
    };

    /**
     * Determine whether the menu is open.
     * @return {Promise.<Boolean>} A promise that will be resolved with the presency of the swipe.
     */
    this.isMenuOpen = function() {
        return element(by.css('.menu-overlay')).isDisplayed();
    };

    /**
     * Closes the menu.
     */
    this.closeMenu = function() {
        this.clickElement(by.css('.menu-overlay'));
    };

    /**
     * On the alert, clicks on 'Ok'.
     */
    this.clickMessageOk = function() {
        this.clickAlertButton("OK");
    };

    /**
     * On the alert, clicks on 'Yes'.
     */
    this.clickMessageYes = function() {
        this.clickAlertButton("Yes");
    };

    /**
     * On the alert, clicks on 'No'.
     */
    this.clickMessageNo = function() {
        this.clickAlertButton("No");
    };

    /**
     * Click on the especified row'.
     * @param {String} columnName- The column of the grid that will have the row value.
     * @param {String} rowValue - The value of the row to be selected.
     * @param {String} [gridId] - The grid's Id.
     * @param {String} [popupPrefix] - The popup's prefix.
     */
    this.clickOnGrid = function(columnName, rowValue, gridId, popupPrefix) {
        gridId = this.validateOptionalParameter(gridId, '');
        popupPrefix = this.validateOptionalParameter(popupPrefix, '');
        var gridSelector = '#grid-' + gridId + '>.body>.body-content>.tr>div[data-zh-field-name="' + columnName + '"]>span';
        gridSelector = popupPrefix + gridSelector;
        this.clickElement(by.cssWithExactText(gridSelector, rowValue));
    };

    /**
     * Click on a especified row on a grid that is inside of a popup.
     * @param {String} columnName - The column of the grid that will have the row value.
     * @param {String} rowValue - The value of the row to be selected.
     * @param {String} [gridId] - The grid's Id.
     */
    this.clickOnGridInsidePopup = function(columnName, rowValue, gridId) {
        this.clickOnGrid(columnName, rowValue, gridId, '.popup-container ');
    };

    /**
     * Click on a specific value inside a select.
     * @param {String} fieldName - The select name.
     * @param {String} columnName - The column of the grid that will have the rowValue.
     * @param {String} rowValue - The value of the row to be selected.
     */
    this.clickOnSelectGrid = function(fieldName, columnName, rowValue) {
        this.clickField(fieldName);
        this.clickOnGrid(columnName, rowValue, '0');
    };

    /**
     * Click on specific row(s) inside a select multiple.
     * @param {String} fieldName - The select multiple name.
     * @param {String} columnName- The column of the grid that will have the rowValues.
     * @param {Array} rowsValues - The value of the rows to be selected.
     */
    this.clickOnSelectMultiple = function(fieldName, columnName, rowsValues) {
        this.clickField(fieldName);
        for (var i in rowsValues) {
            if (rowsValues.hasOwnProperty(i)) {
                this.clickOnGrid(columnName, rowsValues[i], '9999');
            }
        }
        this.clickRightActionByLabel('Ok');
    };

    /**
     * Click on a specific row inside a select list.
     * @param {String} fieldName - The select name.
     * @param {String} itemText - The text of the item that will be selected.
     */
    this.clickOnSelectList = function(fieldName, itemText) {
        this.clickField(fieldName);
        this.clickElement(by.cssWithExactText('.zh-template-list>li.zh-list-item-label>div>div>span', itemText));
    };

    /**
     * Click on a specific row inside a select native.
     * @param {String} fieldName - The select name.
     * @param {String} itemLabel - The text of the item that will be selected.
     */
    this.clickOnSelectNative = function(fieldName, itemLabel) {
        var select = element(by.css("select[id='" + fieldName + "']")).element(by.xpath("..")).element(by.css("div.new-select"));
        this.clickByElementFinder(select);

        var selectItem = select.element(by.cssWithExactText('.list>li', itemLabel));
        this.clickByElementFinder(selectItem);
    };

    /**
     * Click on a specific row inside the grid wizard.
     * @param {String} columnName - The column name.
     * @param {String} rowValue - The column of the grid that will have the rowValue.
     */
    this.clickOnGridWizard = function(columnName, rowValue) {
        this.clickOnGrid(columnName, rowValue, '1');
    };

    /**
     * Click on a specific radio buttom.
     * @param {String} fieldName - The field name.
     * @param {String} radioLabel - The radio's label.
     */
    this.clickRadioButton = function(fieldName, radioLabel) {
        this.clickElement(by.cssContainingText('.zh-radio-group-' + fieldName + '>.zh-radio>.zh-radio-label', radioLabel));
    };

    /**
     * Click on a specific button on view mode.
     * @param {String} fieldName - The field name.
     */
    this.clickOnFieldButton = function(fieldName) {
        this.clickElement(by.css('#span-field-' + fieldName + '>button'));
    };

    /**
     * Click on a specified option on the alert.
     * @param {String} optionLabel - O label of the option.
     */
    this.clickAlertButton = function(optionLabel) {
        this.clickElement(by.cssWithExactText('.zh-footer-alert>button>span', optionLabel));
    };

    /**
     * Click on the field checkbox.
     * @param {String} fieldName - The field name.
     */
    this.clickCheckbox = function(fieldName) {
        this.clickElement(by.css("label[for='zh-checkbox-" + fieldName + "']"));
    };

    /**
     * Click on the right action of the grid.
     * @param {Object} rowFilter - The row filter.
     * @param {String} actionIconName - Action's Icon's name.
     * @param {String} [gridId] - The grid's Id.
     */
    this.clickRightGridAction = function(rowFilter, actionIconName, gridId) {
        gridId = this.validateOptionalParameter(gridId, '');
        var rows = element.all(by.css('#grid-' + gridId + '>.body>.body-content>.tr'));
        for (var columnName in rowFilter) {
            if (rowFilter.hasOwnProperty(columnName)) {
                var columnValue = rowFilter[columnName];
                var columnLocator = by.cssContainingText('div[data-zh-field-name="' + columnName + '"]>span', columnValue);
                rows = rows.all(columnLocator).all(by.xpath('..')).all(by.xpath('..'));
            }
        }
        var rightActions = rows.get(0).element(by.css('.right-grid-actions>.zh-grid-actions'));
        this.clickByElementFinder(rightActions.element(by.css('.zh-icon-' + actionIconName)));
    };

    /**
     * Get the relations value from form.
     * @param {String} fieldName - The form name.
     * @returns {Promise.<String>} A promise which resolves the form's relations' value.
     */
    this.getRelationsFormFieldValue = function(fieldName) {
        return element(by.css(".zh-relationsForm div[name=" + fieldName + "]>.zh-select-text")).getText();
    };

    /**
     * Click on the close icon from the select field.
     * @param {String} fieldName - The select name.
     */
    this.clickIconCloseInSelectField = function(fieldName) {
        var fieldCloseIconPath = element(by.css(".zh-field-" + fieldName + ">.zh-icon-close-x"));
        fieldCloseIconPath.getAttribute('class').then(function(classNames) {
            if (classNames.indexOf('ng-hide') === -1) {
                self.clickByElementFinder(fieldCloseIconPath.element(by.css('svg>#close-x')));
            }
        });
    };

    /**
     * Determine whether the alert is visible on the page.
     * @returns {Promise.<Boolean>} A promise with the result of the alert's visibility.
     */
    this.isAlertVisible = function() {
        return element(by.css("aside[data-zh-alert]")).isDisplayed();
    };

    /**
     * Edits an inline field.
     * @param {(String|Number)} newValue - The value to be inserted.
     * @param {Integer} row - The row's number. Beginning from 0.
     * @param {Integer} column - The columns's number. Beginning from 0.
     * @param {String} gridId - The grid'sw Id.
     * @param {Boolean} [hasCheckbox=false] - Param telling if there are checkboxs on the grid.
     */
    this.editInlineFieldValue = function(newValue, row, column, gridId, hasCheckbox) {
        var gridRowColumnLocator = element(this.getGridRowColumnLocator(gridId, row, column, hasCheckbox));
        var inlineField = gridRowColumnLocator.element(by.css('.editable-content>input'));
        this.clickByElementFinder(gridRowColumnLocator);
        browser.wait(ec.presenceOf(inlineField));
        inlineField.clear();
        inlineField.sendKeys(newValue);
    };

    /**
     * Click on menu label inside other labels.
     * @param {String[]} fullPath - The path of the target window.
     */
    this.clickMenuByPath = function(fullPath) {
        var prefix = '.zh-main-menu>.menu-group>.menu-group-items>';
        var child;
        for (var index in fullPath) {
            if (fullPath.hasOwnProperty(index)) {
                child = element(by.cssWithExactText(prefix + '.menu-item>.menu-link>.menu-title', fullPath[index]));
                child = child.element(by.xpath('..')).element(by.xpath('..'));
                this.clickByElementFinder(child);
                prefix = '.sub-menu>';
            }
            browser.sleep(500);
        }
    };

    /**
     * Get the symbol representing the actual order.
     * @param {String} gridId - The grid's Id.
     * @param {Integer} column - The column's number. Beginning from 0.
     * @returns {String} The symbol representing the actual order.
     */
    this.getColumnOrderStatus = function(gridId, column) {
        var arrowSelector = by.css('div#grid-' + gridId + '>div.header>div.header-content>.tr>div:nth-child(' + (column + 1) + ')>span>span.order-arrow');
        var columnGridArrow = element(arrowSelector);
        return columnGridArrow.isPresent().then(function(isPresent) {
            return isPresent ? columnGridArrow.getText() : ORDER_NONE_STR;
        });
    };

    /**
     * Click to change the order the especified times.
     * @param {String} gridId - The grid's id.
     * @param {Integer} column - The column's number. Beginning from 0.
     * @param {Number} clicks - The number of clicks to be executed.
     */
    this.clickOnColumnName = function(gridId, column, clicks) {
        for (var i = 0; i < clicks; i++) {
            this.clickElement(by.css('div#grid-' + gridId + '>div.header>div.header-content>.tr>div:nth-child(' + (column + 1) + ')>span.zh-ripple-effect'));
        }
    };

    /**
     * Change the order of the grid to the especified.
     * @param {String} gridId - The grid's id.
     * @param {Integer} column - The column's number. Beginning from 0.
     * @param {String} order - The ordenation name. Can asume only ORDER_X consts.
     */
    this.orderGridByColumn = function(gridId, column, order) {
        function clickNTimesOnColumn(nrClicks) {
            self.clickOnColumnName(gridId, column, nrClicks);
        }

        this.getColumnOrderStatus(gridId, column).then(function(text) {
            var nrClicksToAtual = ORDER_SYMBOLS.indexOf(text);
            var nrClicksToDesired = order;
            var nrNecessaryClicks = (nrClicksToDesired + ORDER_SYMBOLS.length - nrClicksToAtual) % ORDER_SYMBOLS.length;
            clickNTimesOnColumn(nrNecessaryClicks);
        });
    };

    /**
     * Uses the menu search.
     * @param {String} searchText - The text to fill the search field.
     */
    this.useMenuSearch = function(searchText) {
        element(by.className('zh-input-search-menu')).sendKeys(searchText);
    };

    /**
     * Removes an selected item from select multiple.
     * @param {String} fieldName - The select-multiple's name.
     * @param {String} itemText - The item's text to be deleted.
     */
    this.removeSelectMultipleItem = function(fieldName, itemText) {
        var selectMutipleSelector = '#' + fieldName + '>div>span';
        element(by.cssWithExactText(selectMutipleSelector, itemText)).click();
    };

    /**
     * Returns the list's row {@link https://github.com/angular/protractor/blob/master/docs/locators.md locator}.
     * @param {Number} row - The number of the row with the desired cdd class list. Beginning from 0.
     * @param {Boolean} [isInsidePopup=false] - Boolean indicating if the element is inside a popup.
     * @returns {Locator} - The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     */
    this.getListRowLocator = function(row, isInsidePopup) {
        var selector = 'div.zh-list-container>ul.zh-template-list>li:nth-child(' + (row + 2) + ')';
        selector = this.setPopupPrefix(selector, isInsidePopup);
        return by.css(selector);
    };

    /**
     * Returns the css class list of the specified row inside a list. The list can be child of some field (e.g. select popup).
     * @param {Number} row - The number of the row with the desired cdd class list. Beginning from 0.
     * @param {Boolean} [isInsidePopup=false] - Boolean indicating if the element is inside a popup.
     * @returns {Array.<String>} The element css classes.
     */
    this.getRowClassesOnList = function(row, isInsidePopup) {
        isInsidePopup = this.validateOptionalParameter(isInsidePopup, false);

        var component = element(this.getListRowLocator(row, isInsidePopup));
        return component.getAttribute("class");
    };

    /**
     * Returns the grid's row {@link https://github.com/angular/protractor/blob/master/docs/locators.md locator}.
     * @param {Number} row - The number of the row with the desired css class list. Beginning from 0.
     * @param {Boolean} [isInsidePopup=false] - Boolean indicating if the element is inside a popup.
     * @param {String} [gridId='9999'] - Grid's id where the row is inside.
     * @returns {Locator} The {@link https://github.com/angular/protractor/blob/master/docs/locators.md element locator}.
     */
    this.getGridRowLocator = function(row, isInsidePopup, gridId) {
        gridId = this.validateOptionalParameter(gridId, '9999');
        var locator = 'div.zh-grid-container>div#grid-' + gridId + '>div.body>div>.tr:nth-child(' + (row + 1) + ')';
        locator = this.setPopupPrefix(locator, isInsidePopup);
        return by.css(locator);
    };

    /**
     * Returns the css class list of the specified row inside a grid. The grid can be child of some field (e.g. select popup).
     * @param {Number} row - The number of the row with the desired css class list. Beginning from 0.
     * @param {Boolean} [isInsidePopup=false] - Boolean indicating if the element is inside a popup.
     * @param {String} [gridId='9999'] - The grid's id.
     * @returns {Array.<String>} The element css classes.
     */
    this.getRowClassesOnGrid = function(row, isInsidePopup, gridId) {
        gridId = this.validateOptionalParameter(gridId, '9999');
        var component = element(this.getGridRowLocator(row, isInsidePopup, gridId));
        return component.getAttribute("class");
    };

    /**
     * Click on a specified item on a list.
     * @param {String} itemText - The text of the item.
     */
    this.clickOnListItem = function(itemText) {
        var itemSelector = by.cssContainingText('.zh-template-list>li.zh-list-item-label>div>div>span', itemText);
        this.clickElement(itemSelector);
    };

    /**
     * Get a specified item of a list.
     * @param {String} itemText - The text of the item.
     */
    var getListItemCheckbox = function(itemText) {
        var itemSelector = by.cssContainingText('.zh-template-list>li.zh-list-item-label>div>div>span', itemText);
        var itemRow = element(itemSelector).element(by.xpath('../../..'));
        var checkboxElement = itemRow.element(by.css('div.zh-checkbox > label'));
        return checkboxElement;
    };

    /**
     * Click the checkbox of a specified item on a list.
     * @param {String} itemText - The text of the item.
     */
    this.clickOnListItemCheckbox = function(itemText) {
        var checkboxElement = getListItemCheckbox(itemText);
        this.clickByElementFinder(checkboxElement);
    };

    /**
     * Returns a promise resolving into a boolean telling wether the checkbox is checked or not.
     * @param {String} itemText - The text of the item.
     */
    this.isListItemChecked = function(itemText) {
        var checkboxElement = getListItemCheckbox(itemText).element(by.css('span.zh-icon'));
        return checkboxElement.getAttribute('class').then(function(classes) {
            return classes.split(' ').indexOf('.ng-hide') === -1;
        });
    };

    /**
     * Get the description order of an especified swipe.
     * @param {Boolean} isInsideAnotherSwipe - Indicates if the Swipe is inside another swipe.
     * @param {Integer} [swipeLevel=2] - The level of the swipe. Beggining from 1.
     * @returns {Promise.<String>} The swipe's descriptionOrder. It's returns is like "children << parent" if there is any hierarchy.
     */
    this.getSwipeDescriptionOrder = function(isInsideAnotherSwipe, swipeLevel) {
        var descriptionOrderSelector;
        var descriptionOrder;

        if (isInsideAnotherSwipe) {
            swipeLevel = swipeLevel ? swipeLevel : 2;
            descriptionOrderSelector = "section.zh-swipe-container.zh-swipe-right.open.multiple-swipes > div:nth-child(" + swipeLevel + ") > div.swipe-parent-description.ng-scope span";
            var descriptionOrderObj = element.all(by.css(descriptionOrderSelector));
            return descriptionOrderObj.getText().then(function(text) {
                descriptionOrder = text.join(" << ");
                return descriptionOrder;
            });
        } else {
            descriptionOrderSelector = ".zh-swipe-container div.swipe-parent-description.ng-scope > span";
            descriptionOrder = element(by.css(descriptionOrderSelector));
            return descriptionOrder.getText();
        }
    };

    /**
     * Get the  field group element especified by label
     * @param   {String} label Field group label
     * @returns {Element}      Field group element
     */
    this.getFieldGroupLabelElement = function(label) {
        return element(by.cssWithExactText('zh-field-group', label));
    };

    /**
     * Open year on currently opened calendar
     *
     * @param  {Integer} year Year to open month list
     */
    this.openCalendarYear = function(year) {
        this.clickElement(by.css('.calendar-heading'));
        this.clickElement(by.css('.calendar-heading'));
        goToPreviousDecade(year);
        goToNextDecade(year);
        this.clickYearCalendar(year);
    };

    /**
     * Open month on currently opened calendar
     *
     * @param  {Integer} month    Month to open days list
     * @param  {Integer} year     Year to open month list
     * @param  {String}  language Language to search for months on calendar
     */
    this.openCalendarMonth = function(month, year, language) {
        var MONTH = this.getMonthsByLanguage(language);
        this.openCalendarYear(year);
        this.clickMonthCalendar(MONTH[month]);
    };

    /**
     * Get the number of rows on the especified tree grid
     *
     * @param   {Integer} treeId The tree-grid's ID
     *
     * @returns {Promise.<Integer>} A promise that will be resolved with the number rows on the tree grid
     */
    this.getNrRowsOnTreeGrid = function(treeId) {
        return element.all(by.css('#grid-' + treeId + '>.body>div>.tr')).count();
    };

    /**
     * Open or close tree row that has field rows
     *
     * @param  {String}  gridId        Grid id
     * @param  {Integer} row           Index of row to toggle
     * @param  {Boolean} isInsidePopup Indicator to especified if the floatingButton is inside a popup.
     */
    this.toggleTreeParentRow = function(gridId, row, isInsidePopup) {
        var locator = 'div.zh-grid-container>div#grid-' + gridId + '>div.body>div>.tr:nth-child(' + (row + 1) + ')';

        element(by.css(locator)).element(by.css('.tree-expand-icon')).click();
    };

    /**
     * Get notification text of specified notification
     *
     * @param   {Integer} index Index of notification to get text
     *
     * @returns {Promise.<String>} Notification text
     */
    this.getNotificationText = function(index) {
        var locator = by.css('.zh-notification-messages li:nth-child(' + (index + 1) + ') .notification-message-text');
        return element(locator).getText();
    };

    /**
     * Retrieve the field value inside a popup.
     * @param {String} fieldName - The field name.
     * @returns {Promise.<String>} A promise that will be solved with the value of the field.
     */
    this.getFieldValueOnPopup = function(fieldName) {
        return element(by.css('.popup-container #' + fieldName + " span")).getText();
    };

    /**
     * Click to open the floating control, or close it. Waiting it to be clickable.
     */
    this.toggleFloatingControl = function() {
        this.clickElement(by.css('.zh-floating-control .control-handle'));
    };

    /*
     * Checks if the floating control is opened.
     * @returns {Promise.<Boolean>} Promise that will be solved telling if the floating control is opened or not.
     */
    this.isFloatingControlOpened = function() {
        return element(by.css('.zh-floating-control.opened')).isPresent();
    };

    /**
     * @deprecated This function has a new name: openfloating control.
     */
    this.openFloatingButton = function(){
      this.openFloatingControl();
    };

    /**
     * Open the floating control, testing if it is already opened.
     * @returns {Promise} A promise that will be solved when the floating control is opened.
     */
    this.openFloatingControl = function() {
        this.isFloatingControlOpened().then(function(isOpened) {
            if (!isOpened) {
                self.toggleFloatingControl();
            }
        });
    };
 
     /**
     * @deprecated This function has a new name: selectFloatingControlAction.
     */
    this.clickFloatingAction = function(actionName){
        this.selectFloatingControlAction(actionName);
    }

    /**
     * Click on an especified action of floating control, waiting it to be clickable.
     * @param {String} actionName - The action to be opened. Can be only 'refresh', 'search', 'layout', 'view'.
     * @param {Boolean} [isOpened=false] - Param telling if the floating control is opened.
     */
    this.selectFloatingControlAction = function(actionName) {
        var actionSelector = 'div.zh-floating-control li.float-action .zh-icon-' + actionName;
        this.clickElement(by.css(actionSelector));
    };

    /**
     * Get the floating control info.
     * @returns {Promise.<String>} A promise that will be solved with the floating control's text.
     */
    this.getFloatingControlInfo = function() {
        return element(by.css(".popup-container .zh-floating-control>.control-info>.text")).getText();
    };

    /**
     * Determine whether the floating control info is present.
     * @returns {Promise.<Boolean>} A promise that will be resolved with the result of the floatingControlInfo's presency.
     */
    this.isFloatingControlInfoPresent = function() {
        return element(by.css(".zh-floating-control.has-status")).isPresent();
    };
	
	this.validationError = function(){
		return element(by.css(".zh-validation-error")).isPresent();
	}
}

//module.exports =  FrameworkFunctionalTestApi;
//module.exports = new FrameworkFunctionalTestApi(brwoser, protractor);
module.exports = new FrameworkFunctionalTestApi();